summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorpatacongo <patacongo@42af7a65-404d-4744-a932-0658087f49c3>2010-06-23 22:00:29 +0000
committerpatacongo <patacongo@42af7a65-404d-4744-a932-0658087f49c3>2010-06-23 22:00:29 +0000
commit1046c6bc457d8ae77b22adfef9e3b4f0b4c2a869 (patch)
tree18ab83ea532c1fb19f5f511e082919a71e2dda26
parent56b5a053b7b0dabcb48601c37962f1886287dbe2 (diff)
downloadnuttx-1046c6bc457d8ae77b22adfef9e3b4f0b4c2a869.tar.gz
nuttx-1046c6bc457d8ae77b22adfef9e3b4f0b4c2a869.tar.bz2
nuttx-1046c6bc457d8ae77b22adfef9e3b4f0b4c2a869.zip
Add SPI-based MMC/SD support
git-svn-id: svn://svn.code.sf.net/p/nuttx/code/trunk@2761 42af7a65-404d-4744-a932-0658087f49c3
-rwxr-xr-xnuttx/configs/nucleus2g/nsh/defconfig4
-rwxr-xr-xnuttx/configs/nucleus2g/src/Makefile7
-rwxr-xr-xnuttx/configs/nucleus2g/src/nucleus2g_internal.h6
-rwxr-xr-xnuttx/configs/nucleus2g/src/up_boot.c10
-rwxr-xr-xnuttx/configs/nucleus2g/src/up_leds.c10
-rwxr-xr-xnuttx/configs/nucleus2g/src/up_nsh.c155
-rwxr-xr-xnuttx/configs/nucleus2g/src/up_ssp.c185
7 files changed, 361 insertions, 16 deletions
diff --git a/nuttx/configs/nucleus2g/nsh/defconfig b/nuttx/configs/nucleus2g/nsh/defconfig
index d53b70a3e..b051ab7ea 100755
--- a/nuttx/configs/nucleus2g/nsh/defconfig
+++ b/nuttx/configs/nucleus2g/nsh/defconfig
@@ -113,7 +113,7 @@ CONFIG_LPC17_UART3=n
CONFIG_LPC17_CAN1=n
CONFIG_LPC17_CAN2=n
CONFIG_LPC17_SPI=n
-CONFIG_LPC17_SSP0=n
+CONFIG_LPC17_SSP0=y
CONFIG_LPC17_SSP1=n
CONFIG_LPC17_I2C0=n
CONFIG_LPC17_I2C1=n
@@ -721,7 +721,7 @@ CONFIG_EXAMPLES_NSH_DISABLEBG=n
CONFIG_EXAMPLES_NSH_ROMFSETC=n
CONFIG_EXAMPLES_NSH_CONSOLE=y
CONFIG_EXAMPLES_NSH_TELNET=n
-CONFIG_EXAMPLES_NSH_ARCHINIT=n
+CONFIG_EXAMPLES_NSH_ARCHINIT=y
CONFIG_EXAMPLES_NSH_IOBUFFER_SIZE=512
CONFIG_EXAMPLES_NSH_DHCPC=n
CONFIG_EXAMPLES_NSH_NOMAC=n
diff --git a/nuttx/configs/nucleus2g/src/Makefile b/nuttx/configs/nucleus2g/src/Makefile
index ad4a9ac9c..78a58d25b 100755
--- a/nuttx/configs/nucleus2g/src/Makefile
+++ b/nuttx/configs/nucleus2g/src/Makefile
@@ -38,9 +38,12 @@
CFLAGS += -I$(TOPDIR)/sched
ASRCS =
-AOBJS = $(ASRCS:.S=$(OBJEXT))
+CSRCS = up_boot.c up_leds.c up_ssp.c
+ifeq ($(CONFIG_EXAMPLES_NSH_ARCHINIT),y)
+CSRCS += up_nsh.c
+endif
-CSRCS = up_boot.c up_leds.c
+AOBJS = $(ASRCS:.S=$(OBJEXT))
COBJS = $(CSRCS:.c=$(OBJEXT))
SRCS = $(ASRCS) $(CSRCS)
diff --git a/nuttx/configs/nucleus2g/src/nucleus2g_internal.h b/nuttx/configs/nucleus2g/src/nucleus2g_internal.h
index b03a6c770..775596e3e 100755
--- a/nuttx/configs/nucleus2g/src/nucleus2g_internal.h
+++ b/nuttx/configs/nucleus2g/src/nucleus2g_internal.h
@@ -118,6 +118,8 @@
#define NUCLEUS2G_5V_ENABLE (GPIO_OUTPUT | GPIO_VALUE_ONE | GPIO_PORT2 | GPIO_PIN13)
#define NUCLEUS2G_5V_DISABLE (GPIO_OUTPUT | GPIO_VALUE_ZERO | GPIO_PORT2 | GPIO_PIN13)
+#define NUCLEUS2G_MMCSD_CS (GPIO_OUTPUT | GPIO_VALUE_ONE | GPIO_PORT0 | GPIO_PIN16)
+
/************************************************************************************
* Public Types
************************************************************************************/
@@ -133,14 +135,14 @@
************************************************************************************/
/************************************************************************************
- * Name: lpc17_spiinitialize
+ * Name: lpc17_sspinitialize
*
* Description:
* Called to configure SPI chip select GPIO pins for the NUCLEUS-2G board.
*
************************************************************************************/
-extern void weak_function lpc17_spiinitialize(void);
+extern void weak_function lpc17_sspinitialize(void);
/************************************************************************************
* Name: lpc17_usbinitialize
diff --git a/nuttx/configs/nucleus2g/src/up_leds.c b/nuttx/configs/nucleus2g/src/up_leds.c
index a3babda36..2d91b52b8 100755
--- a/nuttx/configs/nucleus2g/src/up_leds.c
+++ b/nuttx/configs/nucleus2g/src/up_leds.c
@@ -151,8 +151,6 @@ static bool g_prevled2a;
static bool g_currled2a;
static bool g_prevled2b;
static bool g_currled2b;
-static bool g_prevledhb;
-static bool g_currledhb;
/****************************************************************************
* Private Functions
@@ -267,7 +265,7 @@ void up_led2(uint8_t newstate)
}
/****************************************************************************
- * Name: up_led2
+ * Name: up_ledhb
****************************************************************************/
void up_ledhb(uint8_t newstate)
@@ -283,10 +281,12 @@ void up_ledhb(uint8_t newstate)
case LED_ON:
ledhb = true;
break;
+
+ case LED_NC:
+ return;
}
+
lpc17_gpiowrite(NUCLEUS2G_HEARTBEAT, ledhb);
- g_prevledhb = g_currledhb;
- g_currledhb = newstate;
}
/****************************************************************************
diff --git a/nuttx/configs/nucleus2g/src/up_nsh.c b/nuttx/configs/nucleus2g/src/up_nsh.c
new file mode 100755
index 000000000..3cd3515fe
--- /dev/null
+++ b/nuttx/configs/nucleus2g/src/up_nsh.c
@@ -0,0 +1,155 @@
+/****************************************************************************
+ * config/nucleus2g/src/up_nsh.c
+ * arch/arm/src/board/up_nsh.c
+ *
+ * Copyright (C) 2010 Gregory Nutt. All rights reserved.
+ * Author: Gregory Nutt <spudmonkey@racsa.co.cr>
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in
+ * the documentation and/or other materials provided with the
+ * distribution.
+ * 3. Neither the name NuttX nor the names of its contributors may be
+ * used to endorse or promote products derived from this software
+ * without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+ * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ ****************************************************************************/
+
+/****************************************************************************
+ * Included Files
+ ****************************************************************************/
+
+#include <nuttx/config.h>
+
+#include <stdio.h>
+#include <debug.h>
+#include <errno.h>
+
+#include <nuttx/spi.h>
+#include <nuttx/mmcsd.h>
+
+/****************************************************************************
+ * Pre-Processor Definitions
+ ****************************************************************************/
+
+/* Configuration ************************************************************/
+
+/* PORT and SLOT number probably depend on the board configuration */
+
+#ifdef CONFIG_ARCH_BOARD_NUCLEUS2G
+# define CONFIG_EXAMPLES_NSH_HAVEUSBDEV 1
+# define CONFIG_EXAMPLES_NSH_HAVEMMCSD 1
+# if !defined(CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO) || CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO != 0
+# error "The Nucleus-2G MMC/SD is on SSP0"
+# undef CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO
+# define CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO 0
+# endif
+# if !defined(CONFIG_EXAMPLES_NSH_MMCSDSLOTNO) || CONFIG_EXAMPLES_NSH_MMCSDSLOTNO != 0
+# error "The Nucleus-2G MMC/SD is only one slot (0)"
+# undef CONFIG_EXAMPLES_NSH_MMCSDSLOTNO
+# define CONFIG_EXAMPLES_NSH_MMCSDSLOTNO 0
+# endif
+# ifndef CONFIG_LPC17_SSP0
+# warning "CONFIG_LPC17_SSP0 is not enabled"
+# endif
+#else
+# error "Unrecognized board"
+# undef CONFIG_EXAMPLES_NSH_HAVEUSBDEV
+# undef CONFIG_EXAMPLES_NSH_HAVEMMCSD
+#endif
+
+/* Can't support USB features if USB is not enabled */
+
+#ifndef CONFIG_USBDEV
+# undef CONFIG_EXAMPLES_NSH_HAVEUSBDEV
+#endif
+
+/* Can't support MMC/SD features if mountpoints are disabled */
+
+#if defined(CONFIG_DISABLE_MOUNTPOINT)
+# undef CONFIG_EXAMPLES_NSH_HAVEMMCSD
+#endif
+
+#ifndef CONFIG_EXAMPLES_NSH_MMCSDMINOR
+# define CONFIG_EXAMPLES_NSH_MMCSDMINOR 0
+#endif
+
+/* Debug ********************************************************************/
+
+#ifdef CONFIG_CPP_HAVE_VARARGS
+# ifdef CONFIG_DEBUG
+# define message(...) lib_lowprintf(__VA_ARGS__)
+# else
+# define message(...) printf(__VA_ARGS__)
+# endif
+#else
+# ifdef CONFIG_DEBUG
+# define message lib_lowprintf
+# else
+# define message printf
+# endif
+#endif
+
+/****************************************************************************
+ * Public Functions
+ ****************************************************************************/
+
+/****************************************************************************
+ * Name: nsh_archinitialize
+ *
+ * Description:
+ * Perform architecture specific initialization
+ *
+ ****************************************************************************/
+
+int nsh_archinitialize(void)
+{
+ FAR struct spi_dev_s *ssp;
+ int ret;
+
+ /* Get the SSP port */
+
+ ssp = up_spiinitialize(CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO);
+ if (!ssp)
+ {
+ message("nsh_archinitialize: Failed to initialize SSP port %d\n",
+ CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO);
+ return -ENODEV;
+ }
+
+ message("Successfully initialized SSP port %d\n",
+ CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO);
+
+ /* Bind the SSP port to the slot */
+
+ ret = mmcsd_spislotinitialize(CONFIG_EXAMPLES_NSH_MMCSDMINOR, CONFIG_EXAMPLES_NSH_MMCSDSLOTNO, ssp);
+ if (ret < 0)
+ {
+ message("nsh_archinitialize: Failed to bind SSP port %d to MMC/SD slot %d: %d\n",
+ CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO, CONFIG_EXAMPLES_NSH_MMCSDSLOTNO, ret);
+ return ret;
+ }
+
+ message("Successfuly bound SSP port %d to MMC/SD slot %d\n",
+ CONFIG_EXAMPLES_NSH_MMCSDSPIPORTNO, CONFIG_EXAMPLES_NSH_MMCSDSLOTNO);
+ return OK;
+}
diff --git a/nuttx/configs/nucleus2g/src/up_ssp.c b/nuttx/configs/nucleus2g/src/up_ssp.c
new file mode 100755
index 000000000..1c7e3b5b0
--- /dev/null
+++ b/nuttx/configs/nucleus2g/src/up_ssp.c
@@ -0,0 +1,185 @@
+/************************************************************************************
+ * configs/nucleus2g/src/up_ssp.c
+ * arch/arm/src/board/up_ssp.c
+ *
+ * Copyright (C) 2010 Gregory Nutt. All rights reserved.
+ * Author: Gregory Nutt <spudmonkey@racsa.co.cr>
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in
+ * the documentation and/or other materials provided with the
+ * distribution.
+ * 3. Neither the name NuttX nor the names of its contributors may be
+ * used to endorse or promote products derived from this software
+ * without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+ * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ ************************************************************************************/
+
+/************************************************************************************
+ * Included Files
+ ************************************************************************************/
+
+#include <nuttx/config.h>
+
+#include <stdint.h>
+#include <stdbool.h>
+#include <debug.h>
+
+#include <nuttx/spi.h>
+#include <arch/board/board.h>
+
+#include "up_arch.h"
+#include "chip.h"
+#include "lpc17_internal.h"
+#include "nucleus2g_internal.h"
+
+/* The LM3S6965 Eval Kit microSD CS is on SSI0 */
+
+#if defined(CONFIG_LPC17_SSP0) || defined(CONFIG_LPC17_SSP1)
+
+/************************************************************************************
+ * Definitions
+ ************************************************************************************/
+
+/* Enables debug output from this file (needs CONFIG_DEBUG too) */
+
+#undef SSP_DEBUG /* Define to enable debug */
+#undef SSP_VERBOSE /* Define to enable verbose debug */
+
+#ifdef SSP_DEBUG
+# define sspdbg lldbg
+# ifdef SSP_VERBOSE
+# define sspvdbg lldbg
+# else
+# define sspvdbg(x...)
+# endif
+#else
+# undef SSP_VERBOSE
+# define sspdbg(x...)
+# define sspvdbg(x...)
+#endif
+
+/* Dump GPIO registers */
+
+#ifdef SSP_VERBOSE
+# define ssp_dumpgpio(m) lpc17_dumpgpio(SDCCS_GPIO, m)
+#else
+# define ssp_dumpgpio(m)
+#endif
+
+/************************************************************************************
+ * Private Functions
+ ************************************************************************************/
+
+/************************************************************************************
+ * Public Functions
+ ************************************************************************************/
+
+/************************************************************************************
+ * Name: lpc17_sspinitialize
+ *
+ * Description:
+ * Called to configure SPI chip select GPIO pins for the LM3S6965 Eval Kit.
+ *
+ ************************************************************************************/
+
+void weak_function lpc17_sspinitialize(void)
+{
+ /* Configure the SPI-based microSD CS GPIO */
+
+ ssp_dumpgpio("lpc17_sspinitialize() Entry)");
+
+ /* SSP0 connects only to the MMC/SD slot on the Nucleus 1G board.
+ * P0[15]/TXD1/SCK0/SCK MMC_CLK
+ * P0[16]/RXD1/SSEL0/SSEL MMC_CD
+ * P0[17]/CTS1/MISO0/MISO MMC_DATA0
+ * P0[18]/DCD1/MOSI0/MOSI MMC_MISO
+ *
+ * In SPI mode the MMC/SD DATA3/CD functions as the SPI chip select.
+ */
+
+#ifdef CONFIG_LPC17_SSP0
+ lpc17_configgpio(NUCLEUS2G_MMCSD_CS);
+#endif
+
+ /* SSP1 goes off the Nucleus 2G board to the Babel CAN board along with 3 chip
+ * select pins. However, it is currently not used on that board.
+ */
+
+#ifdef CONFIG_LPC17_SSP1
+# warning "SSP1 chip selects not known"
+#endif
+ ssp_dumpgpio("lpc17_sspinitialize() Exit");
+}
+
+/************************************************************************************
+ * Name: lpc17_ssp0/ssp1select and lpc17_ssp0/ssp1status
+ *
+ * Description:
+ * The external functions, lpc17_ssp0/ssp1select and lpc17_ssp0/ssp1status
+ * must be provided by board-specific logic. They are implementations of the select
+ * and status methods of the SPI interface defined by struct spi_ops_s (see
+ * include/nuttx/spi.h). All other methods (including up_spiinitialize())
+ * are provided by common LPC17xx logic. To use this common SPI logic on your
+ * board:
+ *
+ * 1. Provide logic in lpc17_boardinitialize() to configure SPI/SSP chip select
+ * pins.
+ * 2. Provide lpc17_ssp0/ssp1select() and lpc17_ssp0/ssp1status() functions
+ * in your board-specific logic. These functions will perform chip selection
+ * and status operations using GPIOs in the way your board is configured.
+ * 3. Add a calls to up_spiinitialize() in your low level application
+ * initialization logic
+ * 4. The handle returned by up_spiinitialize() may then be used to bind the
+ * SPI driver to higher level logic (e.g., calling
+ * mmcsd_spislotinitialize(), for example, will bind the SPI driver to
+ * the SPI MMC/SD driver).
+ *
+ ************************************************************************************/
+
+#ifdef CONFIG_LPC17_SSP0
+void lpc17_ssp0select(FAR struct spi_dev_s *dev, enum spi_dev_e devid, bool selected)
+{
+ sspdbg("devid: %d CS: %s\n", (int)devid, selected ? "assert" : "de-assert");
+ ssp_dumpgpio("lpc17_spiselect() Entry");
+
+ if (devid == SPIDEV_MMCSD)
+ {
+ /* Assert the CS pin to the card */
+
+ lpc17_gpiowrite(NUCLEUS2G_MMCSD_CS, !selected);
+ }
+ ssp_dumpgpio("lpc17_spiselect() Exit");
+}
+
+uint8_t lpc17_ssp0status(FAR struct spi_dev_s *dev, enum spi_dev_e devid)
+{
+ sspdbg("Returning SPI_STATUS_PRESENT\n");
+ return SPI_STATUS_PRESENT;
+}
+#endif
+
+#ifdef CONFIG_LPC17_SSP1
+# warning "SSP1 chip selects not known"
+#endif
+
+#endif /* CONFIG_LPC17_SSP0 || CONFIG_LPC17_SSP1 */
diff --git a/nuttx/configs/nucleus2g/src/up_boot.c b/nuttx/configs/nucleus2g/src/up_boot.c
index 36c8ee929..2871a1dd5 100755
--- a/nuttx/configs/nucleus2g/src/up_boot.c
+++ b/nuttx/configs/nucleus2g/src/up_boot.c
@@ -90,14 +90,14 @@ void lpc17_boardinitialize(void)
lpc17_configgpio(NUCLEUS2G_232_POWERSAVE);
#endif
/* Configure SPI chip selects if 1) SPI is not disabled, and 2) the weak function
* lpc17_spiinitialize() has been brought into the link.
*/
-#if defined(CONFIG_LPC17_SPI)
- if (lpc17_spiinitialize)
+#if defined(CONFIG_LPC17_SSP0) || defined(CONFIG_LPC17_SSP0)
+ if (lpc17_sspinitialize)
{
- lpc17_spiinitialize();
+ lpc17_sspinitialize();
}
#endif
@@ -90,14 +90,14 @@ void lpc17_boardinitialize(void)
lpc17_configgpio(NUCLEUS2G_232_POWERSAVE);
#endif
- /* Configure SPI chip selects if 1) SPI is not disabled, and 2) the weak function
- * lpc17_spiinitialize() has been brought into the link.
+ /* Configure SSP chip selects if 1) at least one SSP is enabled, and 2) the weak
+ * function lpc17_sspinitialize() has been brought into the link.
*/